Package: raster
Type: Package
Title: Geographic Data Analysis and Modeling
Version: 3.6-32
Date: 2025-03-27
Imports: Rcpp, methods, terra (>= 1.8-5)
LinkingTo: Rcpp
Depends: sp (>= 1.4-5), R (>= 3.5.0)
Suggests: ncdf4, igraph, tcltk, parallel, rasterVis, MASS, sf,
        tinytest, gstat, fields, exactextractr
Description: Reading, writing, manipulating, analyzing and modeling of spatial data. This package has been superseded by the "terra" package <https://CRAN.R-project.org/package=terra>. 
License: GPL (>= 3)
URL: https://rspatial.org/raster
BugReports: https://github.com/rspatial/raster/issues/
Authors@R: c(
	person("Robert J.", "Hijmans", role = c("cre", "aut"),  
			email = "r.hijmans@gmail.com", 
			comment = c(ORCID = "0000-0001-5872-2872")),
	person("Jacob", "van Etten", role = "ctb"),
	person("Michael", "Sumner", role = "ctb"),
	person("Joe", "Cheng", role = "ctb"),
	person("Dan", "Baston", role = "ctb"),
	person("Andrew", "Bevan", role = "ctb"),
	person("Roger", "Bivand", role = "ctb"),
	person("Lorenzo", "Busetto", role = "ctb"),
	person("Mort", "Canty", role = "ctb"),
	person("Ben", "Fasoli", role = "ctb"),
	person("David", "Forrest", role = "ctb"),
	person("Aniruddha", "Ghosh", role = "ctb"),
	person("Duncan", "Golicher", role = "ctb"),
	person("Josh", "Gray", role = "ctb"),
	person("Jonathan A.", "Greenberg", role = "ctb"),
	person("Paul", "Hiemstra", role = "ctb"),
	person("Kassel", "Hingee", role = "ctb"),
	person("Alex", "Ilich", role = "ctb"),
	person("Institute for Mathematics Applied Geosciences", role="cph"),
	person("Charles", "Karney", role = "ctb"),
	person("Matteo", "Mattiuzzi", role = "ctb"),
	person("Steven", "Mosher", role = "ctb"),
	person("Babak", "Naimi", role = "ctb"),	
	person("Jakub", "Nowosad", role = "ctb"),
	person("Edzer", "Pebesma", role = "ctb"),
	person("Oscar", "Perpinan Lamigueiro", role = "ctb"),
	person("Etienne B.", "Racine", role = "ctb"),
	person("Barry", "Rowlingson", role = "ctb"),
	person("Ashton", "Shortridge", role = "ctb"),
	person("Bill", "Venables", role = "ctb"),
	person("Rafael", "Wueest", role = "ctb")
	)
NeedsCompilation: yes
Packaged: 2025-03-28 04:28:45 UTC; rhijm
Author: Robert J. Hijmans [cre, aut] (<https://orcid.org/0000-0001-5872-2872>),
  Jacob van Etten [ctb],
  Michael Sumner [ctb],
  Joe Cheng [ctb],
  Dan Baston [ctb],
  Andrew Bevan [ctb],
  Roger Bivand [ctb],
  Lorenzo Busetto [ctb],
  Mort Canty [ctb],
  Ben Fasoli [ctb],
  David Forrest [ctb],
  Aniruddha Ghosh [ctb],
  Duncan Golicher [ctb],
  Josh Gray [ctb],
  Jonathan A. Greenberg [ctb],
  Paul Hiemstra [ctb],
  Kassel Hingee [ctb],
  Alex Ilich [ctb],
  Institute for Mathematics Applied Geosciences [cph],
  Charles Karney [ctb],
  Matteo Mattiuzzi [ctb],
  Steven Mosher [ctb],
  Babak Naimi [ctb],
  Jakub Nowosad [ctb],
  Edzer Pebesma [ctb],
  Oscar Perpinan Lamigueiro [ctb],
  Etienne B. Racine [ctb],
  Barry Rowlingson [ctb],
  Ashton Shortridge [ctb],
  Bill Venables [ctb],
  Rafael Wueest [ctb]
Maintainer: Robert J. Hijmans <r.hijmans@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-28 08:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:55:08 UTC; unix
Archs: raster.so.dSYM
