import(methods, Rcpp, terra)
import(terra, except=c(area, gridDistance))
importFrom(sp, spplot, coordinates, wkt, plot, merge, bbox, disaggregate, proj4string)
importFrom(grDevices, terrain.colors, heat.colors, rainbow)
exportClasses(Extent, BasicRaster, Raster, RasterLayer, RasterBrick, RasterStack, RasterStackBrick)
exportMethods("[", "[[", "==", "!=", "!", "%in%", adjacent, aggregate, all.equal, animate, Arith, approxNA, area, as.array, as.character, as.data.frame, as.factor, as.list, as.vector, as.matrix, as.raster, atan2, bandnr, bbox, bind, barplot, blockSize, boundaries, boxplot, buffer, brick, calc, clamp, click, cellStats, clump, colSums, Compare, couldBeLonLat, cover, coordinates, contour, corLocal, cut, crosstab, crop, crs, "crs<-", disaggregate, distance, direction, density, erase, extent, extract, extend, flip, focal, freq, getValues, getValuesBlock, geom, gridDistance, hasValues, hist, head, init, inMemory, interpolate, intersect, image, is.factor, isLonLat, layerize, lines, log, Logic, levels, is.factor, as.factor, asFactor, match, mask, Math, Math2, mean, metadata, merge, modal, mosaic, names, "names<-", ncell, ncol, "ncol<-", nlayers, nrow, "nrow<-", overlay, origin, "origin<-", pairs, persp, plot, plotRGB, predict, proj4string, quantile, RGB, raster, rasterize, ratify, rectify, reclassify, res, "res<-", resample, rotate, rowSums, "$", "$<-", sampleRandom, sampleRegular, sampleStratified, scale, select, stackSelect, setMinMax, setValues, shift, stretch, spplot, subset, subs, summary, Summary, stack, symdif, t, tail, terrain, text, trim, unique, unstack, union, update, xmin, xmax, xres, ymin, ymax, yres, zonal, yFromRow, xFromCol,colFromX, rowFromY, cellFromXY, cellFromRowCol, cellFromRowColCombine, xyFromCell, yFromCell, xFromCell, rowColFromCell,rowFromCell, colFromCell, readStart, readStop,values, "values<-", weighted.mean, which.min, which.max, whiches.min, whiches.max, Which, wkt, writeStart, writeStop, writeValues, writeRaster, zonal, zoom, xmin, xmax, ymin, ymax, "xmin<-", "xmax<-", "ymin<-", "ymax<-")
useDynLib(raster, .registration = TRUE)
exportPattern("^[^\\.\\_]")
