# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.doBilinear <- function(xy, x, y, v) {
    .Call(`_raster_doBilinear`, xy, x, y, v)
}

.broom <- function(d, f, dm, dist, down) {
    .Call(`_raster_broom`, d, f, dm, dist, down)
}

.doCellFromRowCol <- function(nrow, ncol, rownr, colnr) {
    .Call(`_raster_doCellFromRowCol`, nrow, ncol, rownr, colnr)
}

.clamp <- function(d, r, usevals) {
    .Call(`_raster_do_clamp`, d, r, usevals)
}

.edge <- function(d, dim, classes, edgetype, dirs) {
    .Call(`_raster_do_edge`, d, dim, classes, edgetype, dirs)
}

.focal_fun <- function(d, w, dim, fun, naonly) {
    .Call(`_raster_do_focal_fun`, d, w, dim, fun, naonly)
}

.focal_get <- function(d, dim, ngb) {
    .Call(`_raster_do_focal_get`, d, dim, ngb)
}

.focal_sum <- function(d, w, dim, narm, naonly, bemean) {
    .Call(`_raster_do_focal_sum`, d, w, dim, narm, naonly, bemean)
}

.getPolygons <- function(xyv, res, nodes) {
    .Call(`_raster_getPolygons`, xyv, res, nodes)
}

.layerize <- function(d, cls, falsena) {
    .Call(`_raster_layerize`, d, cls, falsena)
}

.availableRAM <- function(ram) {
    .Call(`_raster_availableRAM`, ram)
}

.getMode <- function(values, ties) {
    .Call(`_raster_getMode`, values, ties)
}

.doSpmin <- function(x, y) {
    .Call(`_raster_doSpmin`, x, y)
}

.doSpmax <- function(x, y) {
    .Call(`_raster_doSpmax`, x, y)
}

.ppmin <- function(x, y, narm) {
    .Call(`_raster_ppmin`, x, y, narm)
}

.ppmax <- function(x, y, narm) {
    .Call(`_raster_ppmax`, x, y, narm)
}

.doRowMin <- function(x, narm) {
    .Call(`_raster_doRowMin`, x, narm)
}

.doRowMax <- function(x, narm) {
    .Call(`_raster_doRowMax`, x, narm)
}

.aggregate_get <- function(d, dims) {
    .Call(`_raster_aggregate_get`, d, dims)
}

.aggregate_fun <- function(d, dims, narm, fun) {
    .Call(`_raster_aggregate_fun`, d, dims, narm, fun)
}

.get_area_polygon <- function(d, lonlat) {
    .Call(`_raster_get_area_polygon`, d, lonlat)
}

.point_distance <- function(p1, p2, lonlat, a, f) {
    .Call(`_raster_point_distance`, p1, p2, lonlat, a, f)
}

.distanceToNearestPoint <- function(d, p, lonlat, a, f) {
    .Call(`_raster_distanceToNearestPoint`, d, p, lonlat, a, f)
}

.directionToNearestPoint <- function(d, p, lonlat, degrees, from, a, f) {
    .Call(`_raster_directionToNearestPoint`, d, p, lonlat, degrees, from, a, f)
}

.dest_point <- function(xybd, lonlat, a, f) {
    .Call(`_raster_dest_point`, xybd, lonlat, a, f)
}

.reclassify <- function(d, rcl, dolowest, doright, doleftright, NAonly, NAval) {
    .Call(`_raster_reclassify`, d, rcl, dolowest, doright, doleftright, NAonly, NAval)
}

.terrain <- function(d, dim, res, unit, option, geo, gy) {
    .Call(`_raster_do_terrains`, d, dim, res, unit, option, geo, gy)
}

.doCellFromXY <- function(ncols, nrows, xmin, xmax, ymin, ymax, x, y) {
    .Call(`_raster_doCellFromXY`, ncols, nrows, xmin, xmax, ymin, ymax, x, y)
}

.doXYFromCell <- function(ncols, nrows, xmin, xmax, ymin, ymax, cell) {
    .Call(`_raster_doXYFromCell`, ncols, nrows, xmin, xmax, ymin, ymax, cell)
}

.doFourCellsFromXY <- function(ncols, nrows, xmin, xmax, ymin, ymax, xy, duplicates, isGlobalLonLat) {
    .Call(`_raster_doFourCellsFromXY`, ncols, nrows, xmin, xmax, ymin, ymax, xy, duplicates, isGlobalLonLat)
}

