\name{getData}
\alias{getData}
\alias{ccodes}

\title{Get geographic data }

\description{
This function has been deprecated and does not work anymore.
}

\usage{
getData(...)
ccodes()
}

\arguments{
  \item{...}{arguments}   
}
 
